@@ -79,7 +79,7 @@ struct Document {
 
     bool searchfilter;
 
-    wxHashMapBool tags;
+    std::map<wxString, bool> tags;
 
     int editfilter;
 
@@ -192,8 +192,7 @@ struct Document {
             if (!zos.IsOk()) return _(L"Zlib error while writing file.");
             wxDataOutputStream dos(zos);
             rootgrid->Save(dos);
-            wxHashMapBool::iterator tagit;
-            for (tagit = tags.begin(); tagit != tags.end(); ++tagit) {
+            for (auto tagit = tags.begin(); tagit != tags.end(); ++tagit) {
                 dos.WriteString(tagit->first);
             }
             dos.WriteString(wxEmptyString);
@@ -714,7 +713,8 @@ struct Document {
     const wxChar *Save(bool saveas, bool *success = nullptr) {
         if (!saveas && !filename.empty()) { return SaveDB(success); }
         wxString fn =
-            ::wxFileSelector(_(L"Choose TreeSheets file to save:"), L"", L"", L"cts", L"*.cts",
+            ::wxFileSelector(_(L"Choose TreeSheets file to save:"), L"", L"", L"cts",
+                             L"TreeSheets Files (*.cts)|*.cts|All Files (*.*)|*.*",
                              wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxFD_CHANGE_DIR);
         if (fn.empty()) return _(L"Save cancelled.");  // avoid name being set to ""
         ChangeFileName(fn);
@@ -843,8 +843,9 @@ struct Document {
 
             case A_OPEN: {
                 wxString fn =
-                    ::wxFileSelector(_(L"Please select a TreeSheets file to load:"), L"", L"", L"cts",
-                                     L"*.cts", wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR);
+                    ::wxFileSelector(_(L"Please select a TreeSheets file to load:"), L"", L"",
+                                     L"cts", L"TreeSheets Files (*.cts)|*.cts|All Files (*.*)|*.*",
+                                     wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR);
                 return sys->Open(fn);
             }
 
@@ -865,7 +866,7 @@ struct Document {
             }
 
             case A_NEW: {
-                int size = ::wxGetNumberFromUser(_(L"What size grid would you like to start with?"),
+                int size = (int)::wxGetNumberFromUser(_(L"What size grid would you like to start with?"),
                                                  _(L"size:"), _(L"New Sheet"), 10, 1, 25, sys->frame);
                 if (size < 0) return _(L"New file cancelled.");
                 sys->InitDB(size);
@@ -936,7 +937,7 @@ struct Document {
             }
 
             case A_PRINTSCALE: {
-                printscale = ::wxGetNumberFromUser(
+                printscale = (uint)::wxGetNumberFromUser(
                     _(L"How many pixels wide should a page be? (0 for auto fit)"), _(L"scale:"),
                     _(L"Set Print Scale"), 0, 0, 5000, sys->frame);
                 return nullptr;
@@ -1267,7 +1268,7 @@ struct Document {
                 if (selected.TextEdit()) {
                     selected.Cursor(this, A_DOWN, false, false, dc, true);
                 } else {
-                    selected.EnterEdit(this, 0, c->text.t.Len());
+                    selected.EnterEdit(this, 0, (int)c->text.t.Len());
                     Refresh();
                 }
                 return nullptr;
@@ -1467,7 +1468,7 @@ struct Document {
             case A_IMAGESCN: {
                 if (!c->text.image) return _(L"No image in this cell.");
                 if (k == A_IMAGESCN) {
-                    c->text.image->display_scale = sys->frame->csf;
+                    c->text.image->ResetScale(sys->frame->csf);
                 } else {
                     long v = wxGetNumberFromUser(
                         _(L"Please enter the percentage you want the image scaled by:"), L"%",
@@ -1475,9 +1476,9 @@ struct Document {
                     if (v < 0) return nullptr;
                     auto sc = v / 100.0;
                     if (k == A_IMAGESCP) {
-                        c->text.image->Scale(sc);
+                        c->text.image->BitmapScale(sc);
                     } else {
-                        c->text.image->display_scale /= sc;
+                        c->text.image->DisplayScale(sc);
                     }
                 }
                 c->ResetLayout();
@@ -1486,7 +1487,7 @@ struct Document {
             }
 
             case A_ENTERGRID:
-                if (!c->grid) return NoGrid();
+                if (!c->grid) Action(dc, A_NEWGRID);
                 selected = Selection(c->grid, 0, 0, 1, 1);
                 ScrollOrZoom(dc, true);
                 return nullptr;
@@ -1508,7 +1509,9 @@ struct Document {
                 Cell *pp = c->parent->parent;
                 if (!pp) return _(L"Cannot move this cell up in the hierarchy.");
                 if (pp->grid->xs != 1 && pp->grid->ys != 1)
-                    return _(L"Can only move this cell into an Nx1 or 1xN grid.");
+                    return _(L"Can only move this cell into a Nx1 or 1xN grid.");
+                if (c->parent->grid->xs != 1 && c->parent->grid->ys != 1)
+                    return _(L"Can only move this cell from a Nx1 or 1xN grid.");
                 pp->AddUndo(this);
                 selected = pp->grid->HierarchySwap(c->text.t);
                 pp->ResetChildren();
@@ -1548,7 +1551,7 @@ struct Document {
                 return nullptr;
             case A_SEND:
                 DrawSelect(dc, selected);
-                selected.cursorend = c->text.t.Len();
+                selected.cursorend = (int)c->text.t.Len();
                 DrawSelectMove(dc, selected);
                 return nullptr;
 
@@ -1825,15 +1828,13 @@ struct Document {
     }
 
     void RecreateTagMenu(wxMenu &menu) {
-        wxHashMapBool::iterator tagit;
         int i = A_TAGSET;
-        for (tagit = tags.begin(); tagit != tags.end(); ++tagit) { menu.Append(i++, tagit->first); }
+        for (auto tagit = tags.begin(); tagit != tags.end(); ++tagit) { menu.Append(i++, tagit->first); }
     }
 
     const wxChar *TagSet(int tagno) {
-        wxHashMapBool::iterator tagit;
         int i = 0;
-        for (tagit = tags.begin(); tagit != tags.end(); ++tagit)
+        for (auto tagit = tags.begin(); tagit != tags.end(); ++tagit)
             if (i++ == tagno) {
                 Cell *c = selected.GetCell();
                 if (!c) return OneCell();
